from django import template
from django.conf import settings
from ..ecommerce import CanonicalOrder, plata_orderitem_to_canonical
from django.template.loader import render_to_string

register = template.Library()


class GanalyticsRenderNode(template.Node):
    def __init__(self, dont_track_page='False'):
        self.dont_track_page = template.Variable(dont_track_page)

    def render(self, context):
        try:
            dont_track_page = self.dont_track_page.resolve(context)
        except template.VariableDoesNotExist:
            dont_track_page = False
        analytics_id = getattr(settings, 'GOOGLE_ANALYTICS_ID', None)
        return render_to_string('analytics/google-analytics.html', {
            'analytics_id': analytics_id,
            'trackPageview': not dont_track_page})


@register.tag
def ganalytics(parser, token):
    """
    Usage: {% ganalytics %} or {% ganalytics False %} if you wish to not track the page view and only load analytics code.
    You must define GOOGLE_ANALYTICS_ID = "UA-XXXXXXX-X" in your settings.py
    """
    args = token.split_contents()
    if len(args) == 2:
        return GanalyticsRenderNode(args[1])
    elif len(args) == 1:
        return GanalyticsRenderNode()
    else:
        raise template.TemplateSyntaxError("'ganalytics can't take more than one argument")


@register.inclusion_tag('analytics/google-tag-manager.html')
def google_tag_manager():
    return {'GOOGLE_TAG_MANAGER_ID': getattr(settings, 'GOOGLE_TAG_MANAGER_ID', None)}


@register.inclusion_tag('analytics/google-ecommerce.html')
def ecommerce_track(order):
    """ Usage: {% ecommerce_track order %} where `order` is a order object.
You must call this tag before and in conjunction with {% ganalytics %} in your template.
"""
    return {'order': CanonicalOrder(order.id, order.subtotal,
                                    orderitems=order.items.all(),
                                    converter=plata_orderitem_to_canonical,
                                    shipping_cost=order.shipping
                                    )}



from import_export import resources, fields

from django.utils.translation import ugettext_lazy as _

from allink_essentials.legacy_redirect.models import LegacyLink


class LegacyLinkResource(resources.ModelResource):
    old = fields.Field(column_name=_('Zielseite'), attribute='old')

    class Meta:
        model = LegacyLink
        fields = ['old']
        import_id_fields = ['old']
        skip_unchanged = True

    def skip_row(self, instance, original):
        out = super(LegacyLinkResource, self).skip_row(instance, original)
        # skip urls which we never want to be redirected
        if not out and instance.old in [u'(not set)', u'/']:
            return True
        return out



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class LegacyRedirectConfig(AppConfig):
    name = 'allink_essentials.legacy_redirect'
    verbose_name = _('Legacy Redirect')



# -*- coding: utf-8 -*-
from django import forms
from django.utils.translation import get_language, ugettext_lazy as _

from .config import MailChimpConfig
from helpers import list_members_put, get_status_if_new

config = MailChimpConfig()


class SignupForm(forms.Form):
    email = forms.EmailField(label=_(u'E-Mail'))
    language = forms.CharField(max_length=3, required=False)

    def save(self):
        email = self.cleaned_data['email']
        language = self.cleaned_data['language'] if 'language' in self.cleaned_data else get_language()

        data = {
            'email_address': email,
            'status': 'subscribed',
            'status_if_new': get_status_if_new(),
            'language': language
        }

        if config.merge_vars:
            data.update(config.merge_vars)

        if config.additional_fields:
            if not data.get('merge_fields'):
                data['merge_fields'] = {}
            data.get('merge_fields').update(config.additional_fields)

        list_members_put(data)



from django.conf import settings
from django.http import HttpResponse
try:
    from django.utils.module_loading import import_string as import_by_path
except:
    from django.utils.module_loading import import_by_path
from django.views.generic import FormView

from allink_essentials.mailchimp_api.forms import SignupForm


class SignupView(FormView):
    form_class = SignupForm
    template_name = 'mailchimp_api/signup_form.html'

    def __init__(self, **kwargs):
        super(SignupView, self).__init__(**kwargs)
        if getattr(settings, 'MAILCHIMP_SIGNUP_FORM', False):
            self.form_class = import_by_path(settings.MAILCHIMP_SIGNUP_FORM)

    def form_valid(self, form):
        form.save()
        return HttpResponse('ok')

    def get_context_data(self, **kwargs):
        data = super(SignupView, self).get_context_data(**kwargs)
        data['mailchimp_signup_form'] = data.get('form')
        return data



class CanonicalOrder(object):
    """ A generalized order model used for ecommerce tracking.
        Required args are expected to be non-empty for successful tracking.

        orderitems
            list of items in your order model

        converter
            function returning a CanonicalOrder.OrderItem object from an item in orderitems list
    """
    def __init__(self, id, subtotal, orderitems, converter, affiliation='', tax='', shipping_cost='', city='', state='', country=''):
        super(CanonicalOrder, self).__init__()
        self.__dict__.update(locals())  # set args as ivars
        self.items = [converter(i) for i in orderitems]

    class OrderItem(object):
        """Canonical OrderItem model"""
        def __init__(self, sku, unit_price, quantity, name='', category=''):
            super(self.__class__, self).__init__()
            self.__dict__.update(locals())


def plata_orderitem_to_canonical(item):
    """ Converts a plata.shop.order_model.orderitem object to a CanonicalOrder.OrderItem object by defining an attribute mapping. """
    return CanonicalOrder.OrderItem(sku=item.variation.sku,
                                    name=item.variation,
                                    unit_price=item.unit_price,
                                    quantity=item.quantity,
                                    )



from django.db import models
from django.utils.translation import ugettext_lazy as _

from feincms._internal import monkeypatch_property


def register(cls, admin_cls):
    cls.add_to_class('_og_title', models.CharField(_('Title'), max_length=69, blank=True, help_text=_('Used for og:title. Same as title by default.')))
    cls.add_to_class('_og_description', models.TextField(_('Description'), blank=True, help_text=_('Used for og:description. Same as meta description by default.')))

    @monkeypatch_property(cls)
    def og_title(self):
        if self._og_title:
            return self._og_title
        return self.meta_title

    @monkeypatch_property(cls)
    def og_description(self):
        if self._og_description:
            return self._og_description
        return self.meta_description

    if admin_cls:
        admin_cls.search_fields.extend(['_og_title', '_og_description'])
        admin_cls.add_extension_options(_('Social Fields'), {
            'fields': ('_og_title', '_og_description',),
            'classes': ('collapse',),
        })



from django.core.files.storage import FileSystemStorage

import unicodedata


class ASCIIFileSystemStorage(FileSystemStorage):
    """
    Convert unicode characters in name to ASCII characters.
    """
    def get_valid_name(self, name):
        if not isinstance(name, unicode):
            name = unicode(name)
        name = unicodedata.normalize('NFKD', name).encode('ascii', 'ignore')
        return super(ASCIIFileSystemStorage, self).get_valid_name(name)



import os
from importlib import import_module

from fabric.api import run, execute, env, cd, hide, prefix, local as run_local, require
from fabric.contrib import console
from fabric import utils
from fabric import state
from fabric.colors import magenta, red

__all__ = ('local', 'verbose', 'bootstrap', 'delete_pyc', 'create_virtualenv',
           'deploy', 'git_pull', 'update_requirements', 'compilemessages',
           'collectstatic', 'create_database', 'restart_webapp',
           'restart_celery', 'syncdb', 'migrate', 'setup_celery',
           'dump_database', 'dump_media', 'create_local_symlinks', 'env')

if "VIRTUAL_ENV" not in os.environ:
    raise Exception("$VIRTUAL_ENV not found.")

state.output['running'] = False
state.output['stdout'] = False


def _setup_path(name):
    import sys
    sys.path.insert(0, '.')
    settings = import_module('%s.settings_%s' % (env.project_python, name))
    env.django_settings = settings
    env.environment = name
    for key, value in settings.DEPLOYMENT.items():
        setattr(env, key, value)

    env.project_root = os.path.join(env.root, env.project)
    env.code_root = os.path.join(env.project_root, env.project_python)
    env.settings = '%(project)s.settings_%(environment)s' % env
    env.forward_agent = True
    env.is_local = False


def local():
    env.is_local = True
    import sys
    sys.path.insert(0, '.')
    settings = import_module('%s.settings_development' % env.project_python)
    env.django_settings = settings
    env.project_root = ''
    env.code_root = ''
    env.root = '.'
    env.environment = 'development'

env.deployments = ('production',)


def verbose():
    """enables running and stdout output"""
    state.output['running'] = True
    state.output['stdout'] = True

# ===============
# Public Commands
# ===============


def bootstrap():
    """ initialize remote host environment (virtualenv, deploy, update) """
    require('root', provided_by=env.deployments)

    print magenta("Cloning Repository")
    with cd(env.root):
        run("git clone %s" % env.git_repository)

    # some one time setup things
    with cd(env.project_root):
        if env.git_branch != 'master':
            run('git checkout %s' % (env.git_branch,))
        run('mkdir static')
        run('mkdir media')

    with cd(env.code_root):
        run('ln -sf settings_%s.py settings.py' % env.environment)

    # create virtualenv and install all the requirements
    execute('create_virtualenv')
    execute('update_requirements')
    execute('create_database')
    execute('syncdb')
    execute('migrate')

    print magenta("Load initial data")
    with cd(env.project_root), prefix('source env/bin/activate'):
        run('./manage.py loaddata allink_user.json')

    # only compile messages if locale folder is present
    if os.path.isdir('locale'):
        execute('compilemessages')

    execute('collectstatic')


def delete_pyc():
    print magenta("Delete *.pyc files")
    command = 'find . -name \*.pyc -print0 | xargs -0 rm'
    if env.is_local:
        run_local(command)
    else:
        with cd(env.project_root):
            run(command)


def create_virtualenv():
    """ setup virtualenv on remote host """
    require('project_root', provided_by=env.deployments)
    run('virtualenv --setuptools --prompt="(%s)" %s' % (env.project, os.path.join(env.project_root, 'env')))


def deploy():
    """ updates code base on remote host and restarts server process """
    if not env.is_stage:
        if not console.confirm(red('Are you sure you want to deploy production?', bold=True),
                               default=False):
            utils.abort('Production deployment aborted.')
    with cd(env.project_root):
        result = run('git status', quiet=True)
        result = run('git diff-index --name-only HEAD --', quiet=True)
    if result != '':
        utils.abort(red('There are local changes'))
        utils.abort('end')
    execute('git_pull')
    execute('update_requirements')
    execute('delete_pyc')
    execute('migrate')
    # only compile messages if locale folder is present
    if os.path.isdir(os.path.join(os.path.dirname(__file__), 'locale')):
        execute('compilemessages')
    execute('collectstatic')
    execute('restart_webapp')
    execute('restart_celery')


def git_pull():
    "Updates the repository."
    print magenta("Fetch newest version")
    with cd(env.code_root):
        run("git pull")


def update_requirements():
    """ update external dependencies on remote host """
    require('project_root', provided_by=('local',) + env.deployments)
    print magenta("Update requirements")
    if env.is_local:
        run_local('pip install --requirement REQUIREMENTS_LOCAL')
    else:
        with cd(env.project_root):
            with prefix('source env/bin/activate'):
                run('pip install -U pip')
                run('pip install --requirement REQUIREMENTS_SERVER')


def compilemessages():
    """compiles all translations"""
    print magenta("Compile messages")
    with cd(env.project_root):
        with prefix('source env/bin/activate'):
            run('./manage.py compilemessages')


def collectstatic():
    """runs collectstatic on the remote host"""
    print magenta("Collect static files")
    with cd(env.project_root), prefix('source env/bin/activate'):
        run('./manage.py collectstatic --noinput')


def create_database():
    database_name = env.django_settings.UNIQUE_PREFIX
    print magenta("Create database")
    if env.is_local:
        run_local('mysql --user=$MYSQL_USER -p$MYSQL_PASSWORD -e "CREATE DATABASE %s;"' % database_name)
    else:
        database = env.django_settings.UNIQUE_PREFIX
        run('mysql --user=$MYSQL_USER -p$MYSQL_PASSWORD -e "CREATE DATABASE db_%s;"' % database)


def restart_webapp():
    """ touch wsgi file to trigger reload """
    require('project_root', provided_by=env.deployments)
    print magenta("Restart webapp")
    with cd(env.project_root):
        run('touch apache.wsgi')


def restart_celery():
    """restarts the celery worker"""
    require('project_root', provided_by=env.deployments)
    print magenta("Restart celery")
    run('supervisorctl restart %s' % env.celery_worker)


def syncdb():
    """runs syncdb on the remote host"""
    require('project_root', provided_by=env.deployments)
    print magenta("Syncronize database")
    with cd(env.project_root):
        with prefix('source env/bin/activate'):
            run('./manage.py syncdb --noinput')


def migrate():
    """migrates all apps on remote host"""
    require('project_root', provided_by=env.deployments)
    print magenta("Migrate database")
    with cd(env.project_root), prefix('source env/bin/activate'):
        run('./manage.py migrate')


def setup_celery():
    """create a rabbitmq vhost and user"""
    require('project_root', provided_by=env.deployments)
    run('sudo rabbitmqctl add_user %(rabbitmq_username)s %(rabbitmq_password)s' % env.django_settings.DEPLOYMENT)
    run('sudo rabbitmqctl add_vhost %(rabbitmq_vhost)s' % env.django_settings.DEPLOYMENT)
    run('sudo rabbitmqctl set_permissions -p %(rabbitmq_vhost)s %(rabbitmq_username)s ".*" ".*" ".*"' % env.django_settings.DEPLOYMENT)


# ==============
# Local Commands
# ==============

def dump_database():
    """ resets the local database to the database on the server """
    local_settings = import_module('%s.settings_development' % env.project_python)
    require('project_root', provided_by=env.deployments)
    if not console.confirm(red('Are you sure you want to replace the local database with the %s database data?'
                           % env.environment, bold=True), default=False):
        utils.abort('Reset local database aborted.')
    try:
        with hide('output'):
            run_local('mysql --user=$MYSQL_USER -p$MYSQL_PASSWORD -e "DROP DATABASE %s;"' % (local_settings.UNIQUE_PREFIX))
    except:
        pass
    with hide('output'):
        run_local('mysql --user=$MYSQL_USER -p$MYSQL_PASSWORD -e "CREATE DATABASE %s;"' % (local_settings.UNIQUE_PREFIX))
        run_local('ssh %s@%s "source ~/.profile; mysqldump -u \$MYSQL_USER -p\$MYSQL_PASSWORD db_%s | gzip" | gunzip | mysql -u $MYSQL_USER -p$MYSQL_PASSWORD %s' % (env.django_settings.DEPLOYMENT['user'], env.django_settings.DEPLOYMENT['hosts'][0], env.django_settings.UNIQUE_PREFIX, local_settings.UNIQUE_PREFIX))


def dump_media():
    """ Reset local media from remote host """
    require('project_root', provided_by=env.deployments)
    if not console.confirm(red('Are you sure you want to replace the local media with the %s media data?'
                               % env.environment, bold=True), default=False):
        utils.abort('Reset local media aborted.')
    remote_media = os.path.join(env.project_root, 'media',)
    local_media = os.path.join(os.getcwd(), 'media')
    run_local('rsync --delete --exclude=".gitignore" -rvaz %s@%s:%s/ %s' % (env.user, env.hosts[0], remote_media, local_media))


def create_local_symlinks():
    """creates the local symlinks to settings files and pre-commit hook"""
    if not os.path.isdir("media"):
        os.mkdir("media")
    if not os.path.islink("%s/settings.py" % env.project_python) and os.path.isfile("%s/settings_development.py" % env.project_python):
        os.symlink("settings_development.py", "%s/settings.py" % env.project_python)
    if not os.path.isfile(".git/hooks/pre-commit"):
        os.symlink("../../pre-commit", ".git/hooks/pre-commit")


# sollte zb django>=1.6,<1.7 nicht zerstoeren
# def freeze_requirements():
#     with open("REQUIREMENTS") as file:
#         for line in file:
#             if line.lower().startswith('-e') or line.lower().startswith('http'):
#                 os.system("echo '" + line.rstrip() + "' >> REQUIREMENTS_frozen")
#             else:
#                 pkg = line.rstrip().split('==')[0]
#                 os.system("pip freeze | grep -i ^" + pkg + "== >> REQUIREMENTS_frozen")
#     os.system("mv REQUIREMENTS_frozen REQUIREMENTS")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('legacy_redirect', '0004_legacylink_match_subpages'),
    ]

    operations = [
        migrations.AddField(
            model_name='legacylink',
            name='overwrite',
            field=models.CharField(help_text="Overwrites 'New Link', use for special urls that are not listed there", max_length=255, null=True, verbose_name='Overwrite Link', blank=True),
            preserve_default=True,
        ),
    ]



import sys
from django.conf import settings


class MailChimpConfig:
    def __init__(self):

        apikey = getattr(settings, 'MAILCHIMP_API_KEY', '')

        try:
            parts = apikey.split('-')
            if len(parts) != 2:
                print "This doesn't look like an API Key: " + apikey
                print "The API Key should have both a key and a server name, separated by a dash, like this: abcdefg8abcdefg6abcdefg4-us1"
                sys.exit()

            self.shard = parts[1]
            self.api_root = "https://" + self.shard + ".api.mailchimp.com/3.0/"
        except:
            pass
        # SETTINGS
        self.apikey = apikey
        self.default_list_id = getattr(settings, 'MAILCHIMP_DEFAULT_LIST_ID', None)
        self.signup_form = getattr(settings, 'MAILCHIMP_SIGNUP_FORM', None)
        self.merge_vars = getattr(settings, 'MAILCHIMP_MERGE_VARS', None)
        self.additional_fields = getattr(settings, 'MAILCHIMP_ADDITIONAL_FIELDS', None)
        """
        settings.MAILCHIMP_DOUBLE_OPTIN
        - if set to True a Welcome Email will be sent and the double opt-in process will be used
        - if set to False no Welcome Email will be sent and the user gets subscribed directly to the list
        """
        self.double_optin = getattr(settings, 'MAILCHIMP_DOUBLE_OPTIN', True)



# -*- coding: utf-8 -*-
import requests
from requests.exceptions import ConnectionError, RequestException

from django.contrib import messages
from django.db import models
from django.utils.timezone import now
from django.utils.translation import ugettext, ugettext_lazy as _

from allink_essentials.legacy_redirect.utils import base_url


class LegacyLink(models.Model):
    old = models.CharField(_('Old Link'), max_length=255, unique=True)
    new = models.CharField(_('New Link'), max_length=255)
    overwrite = models.CharField(_('Overwrite Link'), max_length=255, null=True, blank=True, help_text=_('Overwrites \'New Link\', use for special urls that are not listed there'))
    active = models.BooleanField(_('Active'), default=True)
    match_subpages = models.BooleanField(_('Match subpages'), default=False, help_text=_('If True, matches all subpages and redirects them to this link'))
    last_test_result = models.NullBooleanField(_('Result of last test'), default=None, help_text=_('Was the last automatic test successfull? (True = Yes, False = No, None = Not yet tested)'))
    last_test_date = models.DateTimeField(_('Date of last test'), null=True, blank=True)

    class Meta:
        verbose_name = _('Legacy Link')
        verbose_name_plural = _('Legacy Links')

    def __unicode__(self):
        return self.old

    def test_redirect(self, request):
        result = False
        old_url = base_url() + self.old
        new_path = self.overwrite if self.overwrite else self.new
        new_url = base_url() + new_path
        try:
            resp = requests.get(old_url)
        except ConnectionError:
            messages.add_message(
                request,
                messages.ERROR,
                ugettext(u'Can\'t connect to url: %(old_url)s') % {'old_url': old_url}
            )
            result = None
        except RequestException as e:
            messages.add_message(request, messages.ERROR, '%s: %s' % (e, self.old))
            result = None
        else:
            # successfull request, test for history
            if resp.status_code == 200:
                try:
                    # first history entry should be the redirect
                    redir = resp.history[0]
                    if redir.status_code == 301:
                        location = redir.headers.get('Location')
                        # location of redirect has to match
                        # Django 1.9 will send back relative urls
                        # if run via dev server
                        # Both absolute and relative urls should
                        # be accounted for
                        # https://en.wikipedia.org/wiki/HTTP_location
                        if location == new_url or location == new_path:
                            result = True
                except IndexError:
                    pass
        self.last_test_result = result
        self.last_test_date = now()
        self.save()
        return result



from django.conf import settings
from django.views.generic import RedirectView


class FuzzyLanguageRedirectView(RedirectView):
    permanent = True  # don't loose link juice http://www.seomoz.org/learn-seo/redirection
    language_keys = dict(settings.LANGUAGES).keys()

    def get_redirect_url(self, **kwargs):
        """
        Return the URL redirect to. Keyword arguments from the
        URL pattern match generating the redirect request
        are provided as kwargs to this method.
        """
        if not 'HTTP_ACCEPT_LANGUAGE' in self.request.META.keys():
            return '/%s/' % settings.LANGUAGE_CODE
        browser_languages = self.request.META['HTTP_ACCEPT_LANGUAGE'].split(';')[0].split(',')
        # looking for exact match
        for language in browser_languages:
            if language in self.language_keys:
                return '/%s/' % language
        # looking for fuzzy match
        for language in browser_languages:
            for language_key in self.language_keys:
                if language[0:2] == language_key[0:2]:
                    return '/%s/' % language_key
        return '/%s/' % settings.LANGUAGE_CODE



# -*- coding: utf-8 -*-

from django import http
from django.template import Context, Engine, TemplateDoesNotExist, loader
from django.utils import six
from django.utils.translation import get_language
from feincms.module.page.models import Page


# this method is for django version >= 1.9
# for older versions use view.page_not_found

def page_not_found(request, exception, template_name='404.html'):
    """
    Standard 404 view + feincms_page in context. Like this we can display
    navigations in 404 page, without using the add_page_if_missing middleware.
    We don't want to use the middleware, cause the page preview function,
    doe't work correctly with it.
    """
    exception_repr = exception.__class__.__name__
    # Try to get an "interesting" exception message, if any (and not the ugly
    # Resolver404 dictionary)
    try:
        message = exception.args[0]
    except (AttributeError, IndexError):
        pass
    else:
        if isinstance(message, six.text_type):
            exception_repr = message
    try:
        feincms_page = Page.objects.active().filter(language=get_language())[0]
    except IndexError:
        feincms_page = None
    context = {
        'request_path': request.path,
        'exception': exception_repr,
        'feincms_page': feincms_page
    }
    request._feincms_page = feincms_page
    try:
        template = loader.get_template(template_name)
        body = template.render(context, request)
        content_type = None             # Django will use DEFAULT_CONTENT_TYPE
    except TemplateDoesNotExist:
        template = Engine().from_string(
            '<h1>Not Found</h1>'
            '<p>The requested URL {{ request_path }} was not found on this server.</p>')
        body = template.render(Context(context))
        content_type = 'text/html'
    return http.HttpResponseNotFound(body, content_type=content_type)



from __future__ import absolute_import, unicode_literals

from django.db import models
from django.core.validators import MaxLengthValidator
from django.utils.translation import ugettext_lazy as _

from feincms import extensions
from feincms._internal import monkeypatch_property


class Extension(extensions.Extension):
    def handle_model(self):

        self.model.add_to_class('_title_tag', models.CharField(_('Title Tag'), max_length=69, blank=True, help_text=_('Title for browser window and search engines. Same as title by default. Recommended structure "Primary Keyword - Secondary Keyword" (max. 69). Your brand name will be added by default.')))
        self.model.add_to_class('meta_description', models.TextField(_('Meta Description'), max_length=139, blank=True, help_text=_('Meta description for search engines (max 139).'), validators=[MaxLengthValidator(139)]))
        self.model.add_to_class('og_image', models.ImageField(_('og:image'), blank=True, null=True, upload_to='og_images', help_text=_('Preview image when sharing in social networks.')))

        @monkeypatch_property(self.model)
        def title_tag(self):
            if self._title_tag:
                return self._title_tag
            return self.title

    def handle_modeladmin(self, modeladmin):
        modeladmin.search_fields.extend(['_title_tag', 'meta_description'])
        modeladmin.add_extension_options(_('Meta Fields'), {
            'fields': ('_title_tag', 'meta_description', 'og_image',),
        })



import logging
from django.http import UnreadablePostError


class SkipUnreadablePostError(logging.Filter):
    """
    use like shown in your logging configuration

    'filters': {
        'skip_unreadable_posts': {
            '()': 'allink_essentials.logging.filters.SkipUnreadablePostError',
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['skip_unreadable_posts'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    """

    def filter(self, record):
        if record.exc_info:
            exc_type, exc_value = record.exc_info[:2]
            if isinstance(exc_value, UnreadablePostError):
                return False
        return True



# -*- coding: utf-8 -*-
from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from import_export.admin import ImportMixin
from import_export.formats import base_formats

from allink_essentials.legacy_redirect.utils import base_url

from allink_essentials.legacy_redirect.forms import LegacyChangeAdminForm
from allink_essentials.legacy_redirect.models import LegacyLink
from allink_essentials.legacy_redirect.resources import LegacyLinkResource


class LegacyLinkAdmin(ImportMixin, admin.ModelAdmin):
    list_display = ['old', 'new', 'overwrite', 'active', 'match_subpages', 'last_test_result', 'last_test_date', 'manual_test']
    list_editable = ['new', 'overwrite', 'active', 'match_subpages']
    readonly_fields = ['last_test_result', 'last_test_date']
    form = LegacyChangeAdminForm
    resource_class = LegacyLinkResource
    actions = ['auto_test']

    def get_changelist_form(self, request, **kwargs):
        kwargs.setdefault('form', LegacyChangeAdminForm)
        return super(LegacyLinkAdmin, self).get_changelist_form(request, **kwargs)

    def get_import_formats(self):
        """ We only allow csv for
            Google Analytics files
        """
        return [base_formats.CSV]

    def auto_test(self, request, queryset):
        for obj in queryset:
            obj.test_redirect(request)
    auto_test.short_description = _(u'Test redirect')

    def manual_test(self, obj):
        link = base_url() + obj.old
        return format_html('<a class="button" href="%s" target="_blank">Link</a>' % link)
    manual_test.short_description = _(u'Manual testing')

admin.site.register(LegacyLink, LegacyLinkAdmin)



# -*- coding: utf-8 -*-
from django import forms
from django.utils.translation import ugettext_lazy as _

from allink_essentials.legacy_redirect.models import LegacyLink
from allink_essentials.legacy_redirect.utils import choices_from_sitemaps


class LegacyChangeAdminForm(forms.ModelForm):
    new = forms.ChoiceField(label=_('New Link'), choices=[], required=False)

    class Meta:
        model = LegacyLink
        fields = ['old', 'new', 'overwrite', 'active', 'match_subpages']

    def __init__(self, *args, **kwargs):
        super(LegacyChangeAdminForm, self).__init__(*args, **kwargs)
        self.fields['new'].choices = choices_from_sitemaps()



from django.db import models


class SearchQuerySet(models.query.QuerySet):
    def __init__(self, model=None, fields=None, using=None, query=None):
        super(SearchQuerySet, self).__init__(model=model, using=using, query=query)
        self._search_fields = fields

    def search(self, query):

        meta = self.model._meta

        # Get the table name and column names from the model
        # in `table_name`.`column_name` style
        columns = [meta.get_field(name, many_to_many=False).column for name in self._search_fields]
        full_names = ["%s.%s" % (meta.db_table, column) for column in columns]

        # Create the MATCH...AGAINST expressions
        fulltext_columns = ", ".join(full_names)
        match_expr = ("MATCH(%s) AGAINST (%%s IN BOOLEAN MODE)" % fulltext_columns)
        match_expr_relevance = ("MATCH(%s) AGAINST (%%s)" % fulltext_columns)

        prefix_query = ('* ').join(query.split(' ')) + '*'

        # Add the extra SELECT and WHERE options
        return self.extra(select={'relevance': match_expr_relevance, 'relevance2': match_expr},
                        where=[match_expr],
                        params=[prefix_query],
                        select_params=[query, prefix_query],
                        order_by=['-relevance2', '-relevance'])


class SearchManager(models.Manager):
    def __init__(self, fields=None):
        super(SearchManager, self).__init__()
        self._search_fields = fields

    def get_query_set(self):
        if self._search_fields:
            return SearchQuerySet(model=self.model, fields=self._search_fields)
        else:
            return super(SearchManager, self).get_query_set()

    def search(self, query):
        return self.get_query_set().search(query)



from django.db import models
from django.core.validators import MaxLengthValidator
from django.utils.translation import ugettext_lazy as _


class AppMeta(models.Model):
    title_tag = models.CharField(_('Title Tag'), max_length=69, blank=True, help_text=_('Title for browser window and search engines. Same as title by default. Recommended structure "Primary Keyword - Secondary Keyword" (max. 69). Your brand name will be added by default.'))
    meta_description = models.TextField(_('Meta Description'), max_length=139, blank=True, help_text=_('Meta description for search engines (max 139).'), validators=[MaxLengthValidator(139)])
    og_image = models.ImageField(_('og:image'), blank=True, null=True, upload_to='og_images', help_text=_('Preview image when sharing in social networks.'))

    class Meta:
        abstract = True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('legacy_redirect', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='legacylink',
            name='active',
            field=models.BooleanField(default=True, verbose_name='Active'),
            preserve_default=True,
        ),
    ]



import os
import unicodedata
import subprocess

from django.core.files.storage import FileSystemStorage


class LosslessImageCompressStorage(FileSystemStorage):
    """
    Applies lossless image compression to jpeg, gif and png images
    """
    def get_valid_name(self, name):
        if not isinstance(name, unicode):
            name = unicode(name)
        name = unicodedata.normalize('NFKD', name).encode('ascii', 'ignore')
        return super(LosslessImageCompressStorage, self).get_valid_name(name)

    def save(self, name, content):
        path = super(LosslessImageCompressStorage, self).save(name, content)
        full_path = self.path(path)
        extension = os.path.splitext(path)[1].lower()
        if extension in ('.jpeg', '.jpg'):
            # handle jpeg files
            subprocess.call(['jpegtran', '-copy', 'all', '-optimize', '-progressive', '-outfile', full_path, full_path])
        elif extension == '.png':
            # handle png files
            subprocess.call(['optipng', '-o2', full_path, full_path])
        elif extension == '.gif':
            # handle gif files
            subprocess.call(['gifsicle', '-b', '-O3', '--careful', full_path])
        return path



from django.conf import settings
from django.contrib.sites.models import Site
from django.http import HttpResponsePermanentRedirect

try:
    from django.contrib.sites.requests import RequestSite
except:
    # deprecated since Django 1.7, works until 1.9
    from django.contrib.sites.models import RequestSite

from urlparse import urlunparse


class ValidateHostMiddleware(object):
    """
    Redirect every hostname which isn't the primary site to the primary site
    """
    def process_request(self, request):
        if settings.DEBUG:
            return
        request_site = RequestSite(request)
        primary_site = Site.objects.get_current()
        if request_site.domain != primary_site.domain:
            scheme = request.is_secure() and 'https' or 'http'
            url = urlunparse((scheme, primary_site.domain, request.path, None, request.META['QUERY_STRING'], None))
            return HttpResponsePermanentRedirect(url)  # don't loose link juice http://www.seomoz.org/learn-seo/redirection



# -*- coding: utf-8 -*-
from django import http
from django.template import RequestContext, TemplateDoesNotExist, loader, Template
from django.utils.translation import get_language
from feincms.module.page.models import Page


# this method is for django version < 1.9
# for newer versions use view.v2.page_not_found

def page_not_found(request, template_name='404.html'):
    """
    Standard 404 view + feincms_page in context. Like this we can display
    navigations in 404 page, without using the add_page_if_missing middleware.
    We don't want to use the middleware, cause the page preview function,
    doe't work correctly with it.
    """
    try:
        template = loader.get_template(template_name)
        content_type = None             # Django will use DEFAULT_CONTENT_TYPE
    except TemplateDoesNotExist:
        template = Template(
            '<h1>Not Found</h1>'
            '<p>The requested URL {{ request_path }} was not found on this server.</p>')
        content_type = 'text/html'
    try:
        page = Page.objects.active().filter(language=get_language())[0]
    except IndexError:
        page = None
    request._feincms_page = page
    body = template.render(RequestContext(request, {'request_path': request.path, 'feincms_page': page}))
    return http.HttpResponseNotFound(body, content_type=content_type)



"""
Somtimes you need a in_footer Property
This extension lets you do that.
"""

from django.db import models
from django.utils.translation import ugettext_lazy as _

from feincms.admin import tree_editor


def register(cls, admin_cls):
    cls.add_to_class('in_footer', models.BooleanField(_('In footer'), default=False,
        help_text=_('This displays the page in the footer navigation.')))

    setattr(admin_cls, 'in_footer_toggle', tree_editor.ajax_editable_boolean('in_footer', _('in footer')))
    admin_cls.list_display.insert(3, 'in_footer_toggle')



# -*- coding: utf-8 -*-
import os
import random

from fabric.api import run, execute, env, cd, prefix, hide, local as run_local, require
from fabric.contrib import console
from fabric.contrib.files import append, exists
from fabric.colors import green, magenta, red, yellow
from fabric import state
from fabric import utils

if "VIRTUAL_ENV" not in os.environ:
    raise Exception("$VIRTUAL_ENV not found.")

state.output['running'] = False
state.output['stdout'] = False


def _setup_path():
    env.project_root = os.path.join(env.root, env.project)
    env.virtualenv_root = os.path.join(env.project_root, 'env')
    env.settings = '%(project)s.settings.%(environment)s' % env
    env.forward_agent = True
    env.is_local = False


def verbose():
    """enables running and stdout output"""
    state.output['running'] = True
    state.output['stdout'] = True

# ===============
# Public Commands
# ===============


def bootstrap():
    """ initialize remote host environment (virtualenv, deploy, update) """
    require('virtualenv_root', provided_by=env.deployments)

    print magenta("Cloning Repository")
    with cd(env.root):
        run("git clone %s %s" % (env.git_repository, env.project))

    print magenta("Create .env file")
    # add DJANGO_SETTINGS_MODULE to .env
    _add_to_dotenv('DJANGO_SETTINGS_MODULE', '%s.settings.%s' % (env.project_python, env.environment))

    # generate SECRET_KEY
    allowed_chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@$%^&*(-_=+)'
    secret_key = ''.join(random.choice(allowed_chars) for i in range(50)).replace('!', '\\!').replace('&', '\\&').replace('$', '\\$')
    _add_to_dotenv('SECRET_KEY', secret_key)

    # add cache to dotenv
    _add_to_dotenv('CACHE_URL', 'hiredis://127.0.0.1:6379/1/%s' % env.unique_identifier)
    _add_to_dotenv('SESSION_CACHE_URL', 'hiredis://127.0.0.1:6379/2/%s' % env.unique_identifier)

    # some one time setup things
    with cd(env.project_root):
        if env.git_branch != 'master':
            run('git checkout %s' % (env.git_branch,))
        run('mkdir static')
        run('mkdir media')
        run('mkdir tmp')

    # create virtualenv and install all the requirements
    execute('update_requirements')

    execute('create_database')

    execute('migrate')

    print magenta("Load initial data")
    with cd(env.project_root), prefix('source env/bin/activate'):
        run('./manage.py loaddata allink_user.json')

    # only compile messages if locale folder is present
    if os.path.isdir('locale'):
        execute('compilemessages')

    # execute('update_js_requirements')
    execute('collectstatic')
    execute('create_monit_file')


def delete_pyc():
    print magenta("Delete *.pyc files")
    command = 'find . -name \*.pyc -print0 | xargs -0 rm'
    if env.is_local:
        run_local(command)
    else:
        with cd(env.project_root):
            run(command)


def migrate():
    """migrates all apps on the remote host"""
    require('virtualenv_root', provided_by=env.deployments)
    print magenta("Migrate database")
    with cd(env.project_root), prefix('source env/bin/activate'):
        run('./manage.py migrate --noinput')


def deploy():
    """ updates code base on remote host and restarts server process """
    if not env.is_stage:
        if not console.confirm(red('Are you sure you want to deploy production?', bold=True),
                               default=False):
            utils.abort('Production deployment aborted.')
    with cd(env.project_root):
        run('git status', quiet=True)
        result = run('git diff-index --name-only HEAD --', quiet=True)
    if result != '':
        utils.abort(red('There are local changes'))
        utils.abort('end')
    execute('git_pull')
    execute('update_requirements')
    execute('delete_pyc')
    execute('migrate')
    # only compile messages if locale folder is present
    if os.path.isdir(os.path.join(os.path.dirname(__file__), 'locale')):
        execute('compilemessages')
    execute('collectstatic')
    execute('restart_webapp')
    execute('restart_celery')


def dotenv(**kwargs):
    """ adds a key value pair to the .env file on a server """
    require('root', provided_by=env.deployments)
    if not len(kwargs):
        utils.abort('missing variable. usage: fab production dotenv:MYVAR=myvalue')
    for key, value in kwargs.items():
        _add_to_dotenv(key, value)


def create_monit_file():
    require('virtualenv_root', provided_by=env.deployments)
    print magenta("Create Monit File")
    with cd('/home/www-data/etc/monit.d/'):
        run('echo "check process %s_gunicorn with pidfile /home/www-data/projects/%s/tmp/gunicorn.pid" > %s_gunicorn' % (env.unique_identifier, env.project, env.unique_identifier))
        run('echo "\t#!/bin/sh" >> %s_gunicorn' % env.unique_identifier)
        run("echo '\t%s' >> %s_gunicorn" % ('start program = "/home/www-data/projects/%s/startstop.sh start gunicorn"' % env.project, env.unique_identifier))
        run("echo '\t%s' >> %s_gunicorn" % ('stop program = "/home/www-data/projects/%s/startstop.sh stop gunicorn"' % env.project, env.unique_identifier))
        run('echo "\t%s" >> %s_gunicorn' % ("if failed unixsocket /home/www-data/projects/%s/tmp/gunicorn.sock then restart" % env.project, env.unique_identifier))
        run('echo "\tif 5 restarts within 5 cycles then timeout" >> %s_gunicorn' % env.unique_identifier)
        run('echo "\tif totalmemory > 200.0 MB for 5 cycles then alert" >> %s_gunicorn' % env.unique_identifier)
        run('echo "\tif totalmemory > 210.0 MB for 10 cycles then restart" >> %s_gunicorn' % env.unique_identifier)
        run('echo "\tif totalcpu > 50%% for 5 cycles then alert" >> %s_gunicorn' % env.unique_identifier)
        run('echo "\tif totalcpu > 50%% for 10 cycles then restart" >> %s_gunicorn' % env.unique_identifier)
        run('monit reload')


def git_pull():
    "Updates the repository."
    print magenta("Fetch newest version")
    with cd(env.project_root):
        run("git pull")


def update_requirements():
    """update external dependencies on remote host """
    require('root', provided_by=('local',) + env.deployments)
    print magenta("Update requirements")
    if env.is_local:
        run_local('pip install --requirement REQUIREMENTS_LOCAL')
    else:
        _update_requirements_remote()


def compilemessages():
    """compiles all translations"""
    print magenta("Compile messages")
    if env.is_local:
        run_local('./manage.py compilemessages')
    else:
        with cd(env.project_root), prefix('source env/bin/activate'):
            run('./manage.py compilemessages')


def collectstatic():
    """runs collectstatic on the remote host"""
    print magenta("Collect static files")
    with cd(env.project_root), prefix('source env/bin/activate'):
        run('./manage.py collectstatic --noinput')


def _update_requirements_remote():
    with cd(env.project_root):
        result = run('sha1sum --check REQUIREMENTS.sha1', quiet=True)
        if result.return_code is 0:
            print green("No need to update virtualenv.")
            return
        print yellow("Need to create a new virtualenv")
        virtualenv_existed = exists('env')
        if virtualenv_existed:
            # virtualenv exists, build wheels for the new requirements
            with prefix('source env/bin/activate'):
                run('pip install -U pip')
                run('pip install wheel')
                run('pip wheel --requirement REQUIREMENTS_SERVER')
            run('rm -r env_old', quiet=True)
            run('mv env env_old')

        # create new virtualenv
        run('virtualenv env --prompt="(%s)"' % env.project)
        with prefix('source env/bin/activate'):
            run('pip install -U pip')
            run('pip install --requirement REQUIREMENTS_SERVER')
        # create new hash file
        run('sha1sum --tag REQUIREMENTS REQUIREMENTS_SERVER > REQUIREMENTS.sha1')


def update_js_requirements():
    """ update external javascript dependencies on remote host """
    require('root', provided_by=('local',) + env.deployments)
    print magenta("Install javascript requirements")
    if env.is_local:
        run_local('npm install')
    else:
        with cd(env.project_root):
                run('npm install')


def create_database():
    database_name = env.unique_identifier
    print magenta("Create database")
    if env.is_local:
        run_local('psql -U $PGUSER -d postgres -c "CREATE DATABASE %s;"' % database_name)
    else:
        user = env.unique_identifier
        database = env.unique_identifier
        allowed_chars = 'abcdefghijklmnopqrstuvwxyz0123456789'
        password = ''.join(random.choice(allowed_chars) for i in range(10))
        run('sudo nine-manage-databases database create --user=nmd_%s --password=%s nmd_%s' % (user, password, database))
        _add_to_dotenv('DATABASE_URL', 'postgres://nmd_%s:%s@localhost/nmd_%s' % (user, password, database))
        _add_to_dotenv('PG_USER', 'nmd_%s' % (user))
        _add_to_dotenv('PG_PASSWORD', '%s' % (password))


def restart_webapp():
    """ touch wsgi file to trigger reload """
    require('virtualenv_root', provided_by=env.deployments)
    print magenta("Restart webapp")
    with cd(env.project_root):
        run('./startstop.sh restart gunicorn')


# evtl ersetzen durch setup celery -> monit config erstellen
def setup_celery():
    require('virtualenv_root', provided_by=env.deployments)
    user = env.unique_identifier
    vhost = env.unique_identifier
    allowed_chars = 'abcdefghijklmnopqrstuvwxyz0123456789'
    password = ''.join(random.choice(allowed_chars) for i in range(10))
    _add_to_dotenv('BROKER_URL', 'amqp://%s:%s@localhost:5672/%s' % (user, password, vhost))
    _add_to_dotenv('CELERY_RESULT_BACKEND', 'redis://127.0.0.1:6379/0')

    print magenta("Create Monit File")
    with cd('/home/www-data/etc/monit.d/'):
        run('echo "check process %s_celery with pidfile /home/www-data/projects/%s/tmp/celery.pid" > %s_celery' % (env.unique_identifier, env.project, env.unique_identifier))
        run('echo "\t#!/bin/sh" >> %s_celery' % env.unique_identifier)
        run("echo '\t%s' >> %s_celery" % ('start program = "/home/www-data/projects/%s/startstop.sh start celery"' % env.project, env.unique_identifier))
        run("echo '\t%s' >> %s_celery" % ('stop program = "/home/www-data/projects/%s/startstop.sh stop celery"' % env.project, env.unique_identifier))
        run('echo "\tif 5 restarts within 5 cycles then timeout" >> %s_celery' % env.unique_identifier)
        run('echo "\tif totalmemory > 30000 kb for 5 cycles then alert" >> %s_celery' % env.unique_identifier)
        run('echo "\tif totalmemory > 210.0 MB for 10 cycles then restart" >> %s_celery' % env.unique_identifier)
        run('echo "\tif totalcpu > 50%% for 5 cycles then alert" >> %s_celery' % env.unique_identifier)
        run('echo "\tif totalcpu > 50%% for 10 cycles then restart" >> %s_celery' % env.unique_identifier)

    print yellow("Create a rabbitmq user and a rabbitmq vhost in Web GUI (Tip:Lastpass)")
    print yellow("username: %s" % user)
    print yellow("pw: %s" % password)
    print yellow("vhost: %s" % vhost)
    print yellow("Give user (%s) all permissions for vhost (%s)" % (user, vhost))
    print yellow("On server: monit reload")


def restart_celery():
    """restarts the celery worker"""
    require('virtualenv_root', provided_by=env.deployments)
    print magenta("Restart celery")
    with cd(env.project_root):
        run('./startstop.sh restart celery')


# ================
# Private Commands
# ================
def _add_to_dotenv(key, value):
    if isinstance(value, str):
        value = '"%s"' % value
    append(os.path.join(env.project_root, '.env'), '%s=%s' % (key, value))


# ==============
# Local Commands
# ==============
def dump_database():
    require('virtualenv_root', provided_by=env.deployments)
    if not console.confirm(red('Are you sure you want to replace the local database with the %s database data?'
                           % env.environment, bold=True), default=False):
        utils.abort('Reset local database aborted.')
    save_ok_ret_codes = env.ok_ret_codes
    env.ok_ret_codes.append(1)
    with hide('output'):
        run_local('psql -U $PGUSER -d postgres -c "DROP DATABASE %s;"' % (env.project_python,))
    env.ok_ret_codes = save_ok_ret_codes
    run_local('psql -U $PGUSER -d postgres -c "CREATE DATABASE %s;"' % (env.project_python,))
    import time
    timestamp = unicode(time.time()).replace('.', '')
    with cd(env.project_root):
        with prefix('source .env'):
            run("pg_dump --no-privileges --no-owner --no-reconnect --dbname=$DATABASE_URL | gzip > db_dump_%s.sql.gz" % timestamp)
    remote_dump = os.path.join(env.project_root, 'db_dump_%s.sql.gz' % timestamp)
    run_local('rsync -rvaz %s@%s:%s db_dump.sql.gz' % (env.user, env.hosts[0], remote_dump))
    run_local('gunzip db_dump.sql.gz')
    run_local('psql --quiet -U$PGUSER %s < db_dump.sql' % env.project_python)
    run_local('rm db_dump.sql')
    with cd(env.project_root):
        with prefix('source env/bin/activate'):
            run('rm db_dump_%s.sql.gz' % timestamp)


def dump_media():
    """ Reset local media from remote host """
    require('virtualenv_root', provided_by=env.deployments)
    if not console.confirm(red('Are you sure you want to replace the local media with the %s media data?'
                               % env.environment, bold=True), default=False):
        utils.abort('Reset local media aborted.')
    remote_media = os.path.join(env.project_root, 'media',)
    local_media = os.path.join(os.getcwd(), 'media')
    run_local('rsync --delete --exclude=".gitignore" -rvaz %s@%s:%s/ %s' % (env.user, env.hosts[0], remote_media, local_media))


# sollte zb django>=1.6,<1.7 nicht zerstoeren
# def freeze_requirements():
#     with open("REQUIREMENTS") as file:
#         for line in file:
#             if line.lower().startswith('-e') or line.lower().startswith('http'):
#                 os.system("echo '" + line.rstrip() + "' >> REQUIREMENTS_frozen")
#             else:
#                 pkg = line.rstrip().split('==')[0]
#                 os.system("pip freeze | grep -i ^" + pkg + "== >> REQUIREMENTS_frozen")
#     os.system("mv REQUIREMENTS_frozen REQUIREMENTS")


def makemessages(**kwargs):
    """pulls out all strings marked for translation"""
    require('root', provided_by=('local',))
    if not env.is_local:
        utils.abort('runs on local env only. usage: fab local makemessages:lang=fr')
    if 'lang' in kwargs:
        utils.abort('missing language. usage: fab local makemessages:lang=fr')
    print magenta("Make messages")
    cmd = './manage.py makemessages --domain=%s --locale=%s --ignore=env/* --ignore=node_modules/*'
    run_local(cmd % ('django', kwargs['lang']))
    utils.puts('If you have javascript translations, don\'t forget to run:')
    utils.puts(cmd % ('djangojs', kwargs['lang']))



from __future__ import absolute_import, unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from feincms import extensions
from feincms.admin import tree_editor


class Extension(extensions.Extension):
    def handle_model(self):

        self.model.add_to_class('in_footer', models.BooleanField(_('In footer'), default=False, help_text=_('This displays the page in the footer navigation.')))

    def handle_modeladmin(self, modeladmin):
        setattr(type(modeladmin), 'in_footer_toggle', tree_editor.ajax_editable_boolean('in_footer', _('in footer')))
        modeladmin.list_display.insert(3, 'in_footer_toggle')



from django.http import HttpResponsePermanentRedirect
from django.db.models import Q

from allink_essentials.legacy_redirect.models import LegacyLink


class LegacyRedirectMiddleware(object):
    """
    Perma-redirect old links to their new site,
    stripping all GET-params (except google click id)
    """
    def process_request(self, request):
        try:
            link = LegacyLink.objects.get(Q(old=request.path) | Q(old=request.path + '/') | Q(old=request.path[:-1]))

        # check for links which match subpages
        except LegacyLink.DoesNotExist:
            # build up array with path parts
            # e.g. [u'en', u'agency', u'contact']
            path_bits = request.path.split('/')[1:-1]
            length = len(path_bits)
            olds = []

            # omit final bit because we already
            # tested for it in our first try
            # e.g. [u'/en/', u'/en/agency/']
            for i in range(1, length):
                olds.append('/%s/' % ('/').join(path_bits[:i]))

            # if we got multiple links with match_subpages
            # enabled (e.g. /en/ and /en/agency/), return
            # the longer one
            link = LegacyLink.objects.filter(old__in=olds, match_subpages=True).order_by('old').last()

        if not link or not link.active:
            return

        # 'overwrite' takes priority over 'new'
        if link.overwrite:
            new_link = link.overwrite
        elif link.new:
            new_link = link.new
        else:
            return

        if 'gclid' in request.GET:  # preserve Google Click Identifier
            new_link += '?gclid=%s' % request.GET['gclid']
        return HttpResponsePermanentRedirect(new_link)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('legacy_redirect', '0002_legacylink_active'),
    ]

    operations = [
        migrations.AddField(
            model_name='legacylink',
            name='last_test_date',
            field=models.DateTimeField(null=True, verbose_name='Date of last test', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='legacylink',
            name='last_test_result',
            field=models.NullBooleanField(default=None, help_text='Was the last automatic test successfull? (True = Yes, False = No, None = Not yet tested)', verbose_name='Result of last test'),
            preserve_default=True,
        ),
    ]



from django.db import models
from django.core.validators import MaxLengthValidator
from django.utils.translation import ugettext_lazy as _

from feincms._internal import monkeypatch_property


def register(cls, admin_cls):
    cls.add_to_class('_title_tag', models.CharField(_('Title Tag'), max_length=69, blank=True, help_text=_('Title for browser window and search engines. Same as title by default. Recommended structure "Primary Keyword - Secondary Keyword" (max. 69). Your brand name will be added by default.')))
    cls.add_to_class('meta_description', models.TextField(_('Meta Description'), max_length=139, blank=True, help_text=_('Meta description for search engines (max 139).'), validators=[MaxLengthValidator(139)]))
    cls.add_to_class('og_image', models.ImageField(_('og:image'), blank=True, null=True, upload_to='og_images', help_text=_('Preview image when sharing in social networks.')))

    @monkeypatch_property(cls)
    def title_tag(self):
        if self._title_tag:
            return self._title_tag
        return self.title

    if admin_cls:
        admin_cls.search_fields.extend(['_title_tag', 'meta_description'])
        admin_cls.add_extension_options(_('Meta Fields'), {
            'fields': ('_title_tag', 'meta_description', 'og_image',),
        })



#! /usr/bin/env python
from setuptools import setup

import allink_essentials
setup(
    name='allink_essentials',
    version=allink_essentials.__version__,
    description='collection of code fragments',
    long_description='collection of code fragments',
    author='Leandra Finger, Marc Egli',
    author_email='itcrowd@allink.ch',
    url='http://github.com/allink/allink-essentials/',
    license='BSD License',
    platforms=['OS Independent'],
    packages=[
        'allink_essentials',
        'allink_essentials.analytics',
        'allink_essentials.analytics.templatetags',
        'allink_essentials.fabfiles',
        'allink_essentials.feincms_extensions',
        'allink_essentials.logging',
        'allink_essentials.in_footer',
        'allink_essentials.in_footer.templatetags',
        'allink_essentials.mailchimp_api',
        'allink_essentials.middleware',
        'allink_essentials.storage',
        'allink_essentials.view',
        'allink_essentials.view.v2',
        'allink_essentials.search',
        'allink_essentials.mixins',
        'allink_essentials.legacy_redirect',
        'allink_essentials.legacy_redirect.migrations',
    ],
    # package_data={'allink_essentials':'templates/*.html'},
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Communications :: Email',
    ],
    requires=[
    ],
    include_package_data=True,
)



import os
import datetime
from importlib import import_module
import subprocess

from fabric import utils
from fabric.api import run, env, cd, prefix, local as run_local, require
from fabric.context_managers import lcd
from fabric.contrib import console
from fabric.operations import get

if "VIRTUAL_ENV" not in os.environ:
    raise Exception("$VIRTUAL_ENV not found.")


def _setup_path(name):
    import sys
    sys.path.insert(0, '.')
    settings = import_module('settings_%s' % name)
    env.django_settings = settings
    env.environment = name
    for key, value in settings.DEPLOYMENT.items():
        setattr(env, key, value)

    env.project_root = os.path.join(env.root, env.project)
    env.code_root = os.path.join(env.project_root, env.project_python)
    env.virtualenv_root = os.path.join(env.project_root, 'env')
    env.settings = 'settings_%(environment)s' % env
    env.forward_agent = True


def bootstrap():
    """ initialize remote host environment (virtualenv, deploy, update) """
    require('root', provided_by=('staging', 'production'))
    git_clone()
    git_checkout_branch()
    create_virtualenv()
    create_library_symlinks()
    update_requirements()
    create_symlinks()


def create_virtualenv():
    """ setup virtualenv on remote host """
    require('virtualenv_root', provided_by=('staging', 'production'))
    args = '--setuptools'
    run('virtualenv %s %s' % (args, env.virtualenv_root))


def git_pull():
    "Updates the repository."
    require('code_root', provided_by=('staging', 'production'))
    with cd(env.code_root):
        run("git pull")


def git_clone():
    "Updates the repository."
    require('root', provided_by=('staging', 'production'))
    with cd(env.root):
        run("git clone %s %s" % (env.git_repository, env.project))


def git_checkout_branch():
    "Checks out the correct branch."
    require('root', provided_by=('staging', 'production'))
    if env.git_branch != 'master':
        with cd(env.project_root):
            run('git checkout %s' % (env.git_branch,))


def git_reset():
    "Resets the repository to specified version."
    run("cd ~/git/$(repo)/; git reset --hard $(hash)")


def deploy():
    """ updates code base on remote host and restarts server process """
    if not env.is_stage:
        if not console.confirm('Are you sure you want to deploy production?',
                               default=False):
            utils.abort('Production deployment aborted.')
    git_pull()
    touch()


def deployfull():
    """ updates code base on remote host and restarts server process """
    if not env.is_stage:
        if not console.confirm('Are you sure you want to deploy production?',
                               default=False):
            utils.abort('Production deployment aborted.')
    git_pull()
    migrate()
    collectstatic()
    touch()


def update_requirements():
    """ update external dependencies on remote host """
    require('code_root', provided_by=('staging', 'production'))
    with cd(env.project_root):
        with prefix('source env/bin/activate'):
            run('pip install --requirement %s/REQUIREMENTS_SERVER' % env.project_python)


def manage(command):
    """runs manage.py <command> on the remote host"""
    require('code_root', provided_by=('staging', 'production'))
    with cd(env.code_root):
        with prefix('source ../env/bin/activate'):
            run('./manage.py %s' % (manage, command))


def syncdb():
    """runs syncdb on the remote host"""
    require('code_root', provided_by=('staging', 'production'))
    with cd(env.code_root):
        with prefix('source ../env/bin/activate'):
            run('./manage.py syncdb --noinput')


def migrate():
    """migrates all apps on the remote host"""
    require('code_root', provided_by=('staging', 'production'))
    with cd(env.code_root):
        with prefix('source ../env/bin/activate'):
            run('./manage.py migrate')


def compilemessages():
    """compiles all translations"""
    require('code_root', provided_by=('staging', 'production'))
    with cd(env.code_root):
        with prefix('source ../env/bin/activate'):
            run('./manage.py compilemessages')


def create_symlinks():
    """ create settings feincms and admin media links"""
    require('code_root', provided_by=('staging', 'production'))
    with cd(env.project_root):
        run('mkdir static/')
    with cd(env.code_root):
        run('ln -sf settings_%s.py settings.py' % env.environment)


def touch():
    """ touch wsgi file to trigger reload """
    require('code_root', provided_by=('staging', 'production'))
    with cd(env.project_root):
        run('touch apache.wsgi')


def collectstatic():
    """runs collectstatic on the remote host"""
    require('code_root', provided_by=('staging', 'production'))
    with cd(env.code_root):
        with prefix('source ../env/bin/activate'):
            run('./manage.py collectstatic --noinput')


def reset_local_database():
    """ resets the local database to the database on the server """
    require('code_root', provided_by=('staging', 'production'))
    if not console.confirm('Are you sure you want to replace the local database with the %s database data?'
                           % env.environment, default=False):
        utils.abort('Reset local database aborted.')
    filename = "tmp_dump%d_%d_%d.json" % datetime.datetime.now().isocalendar()
    require('code_root', provided_by=('staging', 'production'))
    server_data = os.path.join(env.project_root, filename)
    local_manage = os.path.join(os.getcwd(), 'manage.py')
    local_data = os.path.join(os.getcwd(), filename)
    with cd(env.code_root):
        with prefix('source ../env/bin/activate'):
            run('./manage.py dumpdata > %s' % (server_data,))
        get(server_data, local_data)
        run('rm %s' % server_data)
    with lcd(os.path.dirname(__file__)):
        run_local('%s sqlflush | %s dbshell' % (local_manage, local_manage))
        run_local('%s loaddata %s' % (local_manage, local_data,))
        run_local('rm %s' % local_data)


def reset_local_media():
    """ Reset local media from remote host """
    require('root', provided_by=('staging', 'production'))
    if not console.confirm('Are you sure you want to replace the local media with the %s media data?'
                           % env.environment, default=False):
        utils.abort('Reset local media aborted.')
    remote_media = os.path.join(env.code_root, 'media',)
    local_media = os.path.join(os.getcwd(), 'media/')
    run_local('rsync --delete --exclude=".gitignore" -rvaz %s@%s:%s/ %s' % (env.user, env.hosts[0], remote_media, local_media))


def create_local_symlinks():
    """creates the local symlinks to settings files and pre-commit hook"""
    if not os.path.isdir("media"):
        os.mkdir("media")
    if not os.path.islink("%s/settings.py" % env.project_python) and os.path.isfile("%s/settings_development.py" % env.project_python):
        os.symlink("settings_development.py", "%s/settings.py" % env.project_python)


def create_library_symlinks():
    """used on ubuntu servers to compile pil"""
    require('root', provided_by='production')
    with cd(env.project_root):
        for library in ('libfreetype.so', 'libjpeg.so', 'libz.so'):
            run("ln -s /usr/lib/`uname -i`-linux-gnu/%s env/lib/" % library)


def update_local_requirements():
    """installs local requirements"""
    subprocess.call(["pip", "install", "--requirement", "REQUIREMENTS_LOCAL"])


def freeze_requirements():
    with open("REQUIREMENTS") as file:
        for line in file:
            if line.lower().startswith('-e') or line.lower().startswith('http'):
                os.system("echo '" + line.rstrip() + "' >> REQUIREMENTS_frozen")
            else:
                pkg = line.rstrip().split('==')[0]
                os.system("pip freeze | grep -i ^" + pkg + "== >> REQUIREMENTS_frozen")
    os.system("mv REQUIREMENTS_frozen REQUIREMENTS")



from django import template
from django.utils import translation

from feincms.module.page.models import Page
from feincms.utils.templatetags import SimpleAssignmentNodeWithVarAndArgs
from feincms.utils.templatetags import do_simple_assignment_node_with_var_and_args_helper

register = template.Library()


class FooterNavigationNode(SimpleAssignmentNodeWithVarAndArgs):
    """
    DEPRICATED: USE footer_pages TAG
    {% load in_footer %}
    {% footer_navigation for feincms_page as footer_pages %}
    {% for p in footer_pages %}
        <a href="{{p.get_absolute_url}}">{{p.title}}</a>{% if not forloop.last %} |{% endif %}
    {% endfor %}
    """
    def what(self, page, args, default=None):
        queryset = Page.objects.active().filter(in_footer=True)
        return queryset.filter(language=translation.get_language()) if 'language' in [f.name for f in Page._meta.fields] else queryset

register.tag('footer_navigation', do_simple_assignment_node_with_var_and_args_helper(FooterNavigationNode))


@register.simple_tag(name="footer_pages")
def in_footer():
    """
    {% load in_footer %}
    {% footer_pages as footer_pages %}
    {% for p in footer_pages %}
        <li>
            <a href="{{ p.get_absolute_url }}">{{ p.title }}</a>
        </li>
    {% endfor %}
    """
    queryset = Page.objects.active().filter(in_footer=True)
    return queryset.filter(language=translation.get_language()) if 'language' in [f.name for f in Page._meta.fields] else queryset



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('legacy_redirect', '0003_auto_20160224_1216'),
    ]

    operations = [
        migrations.AddField(
            model_name='legacylink',
            name='match_subpages',
            field=models.BooleanField(default=False, help_text='If True, matches all subpages and redirects them to this link', verbose_name='Match subpages'),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from django.conf import settings
from django.utils import translation

from django.contrib.sites.models import Site
from importlib import import_module

_base_url = None


def choices_from_sitemaps():
    ''' Build up select choices
        from items declared in sitemaps

        - If we got a MPTT object (https://github.com/django-mptt/django-mptt)
        display indentation according to level
        - Display page language where we can get to it
        - Handle i18n sitemaps correctly
    '''
    url_mod = import_module(settings.ROOT_URLCONF)
    sitemaps = url_mod.sitemaps

    def label_from_instance(instance, lang=None):
        if hasattr(instance, '_mptt_meta'):
            # we got a tree object (e.g. FeinCMS page)
            level = getattr(instance, instance._mptt_meta.level_attr)
        else:
            # fallback, no different levels
            level = 0

        name = instance.__unicode__()
        if level:
            level_indicator = '---' * level
            name = u'%s %s' % (level_indicator, name)

        if lang:
            name = u'%s (%s)' % (name, lang)
        elif hasattr(instance, 'language'):
            name = u'%s (%s)' % (name, getattr(instance, 'language'))
        return name

    def _urls(item, smap):
        # copied from django.contrib.sitemaps.get_urls()
        # cycle trough all languages for i18n sensitive
        # sitemaps
        if getattr(smap, 'i18n', False):
            out = []
            current_lang_code = translation.get_language()
            for lang_code, lang_name in settings.LANGUAGES:
                translation.activate(lang_code)
                out += [(item.get_absolute_url(), label_from_instance(item, lang=lang_code))]
            translation.activate(current_lang_code)
        else:
            out = [(item.get_absolute_url(), label_from_instance(item))]
        return out

    urls = [(None, '----------')]
    for name, smap in sitemaps.iteritems():
        if callable(smap):
            urls += [(None, '')]
            urls += [(None, name.upper())]
            urls += [(None, '----------')]
            smap = smap()
            for item in smap.items():
                urls += _urls(item, smap)
    return urls


def base_url():
    global _base_url
    if not '_base_url' not in locals() or not _base_url:
        _base_url = 'http://' + Site.objects.get_current().domain
    return _base_url



from urlparse import urlunparse

from django.conf import settings
from django.http import HttpResponsePermanentRedirect


class EnforceHTTPSMiddleware(object):
    def process_request(self, request):
        if settings.DEBUG or request.is_secure():
            return

        try:
            from django.contrib.sites.requests import RequestSite
        except ImportError:
            # deprecated since Django 1.7, works until 1.9,
            from django.contrib.sites.models import RequestSite

        request_site = RequestSite(request)
        url = urlunparse(('https', request_site.domain, request.path, None, request.META['QUERY_STRING'], None))
        return HttpResponsePermanentRedirect(url)
